@@ -53,7 +53,7 @@ test.data.read_write <- function()
 		}
 
 		# create a new gds file
-		gfile <- createfn.gds("tmp.gds")
+		gfile <- createfn.gds("tmp.gds", allow.duplicate=TRUE)
 
 		# append data
 		node <- add.gdsn(gfile, "data", val=dta)
@@ -62,7 +62,8 @@ test.data.read_write <- function()
 		for (i in 1:200)
 		{
 			i.row <- rows[i]; i.col <- cols[i]
-			r2.dta[i.row, i.col] <- read.gdsn(node, start=c(i.row, i.col), count=c(1, 1))
+			r2.dta[i.row, i.col] <- read.gdsn(node, start=c(i.row, i.col),
+				count=c(1, 1))
 		}
 		for (i in 1:200)
 		{
@@ -70,6 +71,10 @@ test.data.read_write <- function()
 			write.gdsn(node, 0, start=c(i.row, i.col), count=c(1, 1))
 		}
 
+		closefn.gds(gfile)
+		gfile <- openfn.gds("tmp.gds", allow.duplicate=TRUE)
+		node <- index.gdsn(gfile, "data")
+
 		checkEquals(r2.dta, r.dta, sprintf("data random read: %s", n))
 		checkEquals(read.gdsn(node), w.dta, sprintf("data random write: %s", n))
 
@@ -103,11 +108,15 @@ test.data.read_write.compress.zip <- function()
 		}
 
 		# create a new gds file
-		gfile <- createfn.gds("tmp.gds")
+		gfile <- createfn.gds("tmp.gds", allow.duplicate=TRUE)
 
 		# append data
 		node <- add.gdsn(gfile, "data", val=dta, compress="ZIP", closezip=TRUE)
 		
+		closefn.gds(gfile)
+		gfile <- openfn.gds("tmp.gds", allow.duplicate=TRUE)
+		node <- index.gdsn(gfile, "data")
+
 		r2.dta <- matrix(0, nrow=nrow(dta), ncol=ncol(dta))
 		for (i in 1:200)
 		{
@@ -132,7 +141,7 @@ test.data.read_write.file <- function()
 	fn <- sprintf("%s/valid/standard.RData", gdsfmt.path)
 
 	# create a new gds file
-	gfile <- createfn.gds("tmp.gds")
+	gfile <- createfn.gds("tmp.gds", allow.duplicate=TRUE)
 
 	node <- addfile.gdsn(gfile, "tmp.RData", filename=fn)
 	getfile.gdsn(node, "tmp.RData")
@@ -157,7 +166,7 @@ test.data.read_selection <- function()
 		dta <- matrix(valid.dta[[sprintf("valid1.%s", n)]], nrow=50, ncol=40)
 
 		# create a new gds file
-		gfile <- createfn.gds("tmp.gds")
+		gfile <- createfn.gds("tmp.gds", allow.duplicate=TRUE)
 
 		# append data
 		node <- add.gdsn(gfile, "data", val=dta)
